
/*
 *	GSMPClient.cpp
 */

#include "GSMPClient.h"

#define MAX_QUEUE_LEN 20000

extern int g_nSeqNo;

CGSMPClient::CGSMPClient()
{
	pthread_mutex_init(&m_VecDel_mutex, NULL);
	pthread_mutex_init(&m_VecDelResp_mutex, NULL);
	pthread_mutex_init(&m_VecSub_mutex, NULL);
	pthread_mutex_init(&m_VecSubResp_mutex, NULL);
	pthread_mutex_init(&m_VecRpt_mutex, NULL);
	pthread_mutex_init(&m_VecRptResp_mutex, NULL);

	pthread_mutex_init(&m_connect_mutex, NULL);
	pthread_mutex_init(&m_recv_mutex, NULL);
	pthread_mutex_init(&m_send_mutex, NULL);

	m_pLog = NULL;
	m_pXmlConf = NULL;
	m_pkg = NULL;

	m_tLastActive = time(NULL);//һηʱ
	m_nSubmitTimes = 0; //طSUBMITĴ
	m_bInitConf = false;
	m_bTermitnate = false;
	
	m_nSock = 0;
	m_nModel = 0;

	m_strIP = "";
	m_nPort = 0;
	m_strUser = "";
	m_strPass = "";
}

CGSMPClient::~CGSMPClient()
{
	if (m_pLog != NULL)
		delete m_pLog;

	if (this->m_pXmlConf != NULL)
		delete m_pXmlConf;

	if (m_pkg != NULL)
		delete m_pkg;
	
	if (m_nSock > 0)
		close(m_nSock);

	ClearAllQueue();
}

void CGSMPClient::ClearAllQueue(void)
{
	//send deliver
	vector<CGSMPDeliver * > ::iterator itDeliver;
	CGSMPDeliver * pDeliver = NULL;
	pthread_mutex_lock(&m_VecDel_mutex);
	for(itDeliver = m_pDeliver.begin();itDeliver != m_pDeliver.end(); itDeliver++)
	{
		if((*itDeliver) == NULL)
		{
	   		continue;
		}
		pDeliver = *itDeliver;
		delete  (CGSMPDeliver *)pDeliver;
	}
	m_pDeliver.clear();	
	pthread_mutex_unlock(&m_VecDel_mutex);

	//recv submit;
	vector<CGSMPSubmit * > ::iterator itSubmit;
	CGSMPSubmit * pSubmit = NULL;
	pthread_mutex_lock(&m_VecSub_mutex);
	for(itSubmit = m_pSubmit.begin();itSubmit != m_pSubmit.end(); itSubmit++)
	{
		if((*itSubmit) == NULL)
		{
	   		continue;
		}
		pSubmit = *itSubmit;
		delete (CGSMPSubmit *)pSubmit;
	}
	m_pSubmit.clear();	
	pthread_mutex_unlock(&m_VecSub_mutex);

	//send report
	vector<CGSMPReport *> ::iterator itReport;
	CGSMPReport * pReport = NULL;
	pthread_mutex_lock(&m_VecRpt_mutex);
	for(itReport = m_pReport.begin();itReport != m_pReport.end(); itReport++)
	{
		if((*itReport) == NULL)
		{
	   		continue;
		}
		pReport = *itReport;
		delete (CGSMPReport*)pReport;
	}
	m_pReport.clear();	
	pthread_mutex_unlock(&m_VecRpt_mutex);

	//recv report resp
	vector<CGSMPResp *> ::iterator itRptResp;
	CGSMPResp * pRptResp = NULL;
	pthread_mutex_lock(&m_VecRptResp_mutex);
	for(itRptResp = m_pRptResp.begin();itRptResp != m_pRptResp.end(); itRptResp++)
	{
		if((*itRptResp) == NULL)
		{
	   		continue;
		}
		pRptResp = *itRptResp;
		delete  (CGSMPResp *)pRptResp;
	}
	m_pRptResp.clear();	
	pthread_mutex_unlock(&m_VecRptResp_mutex);

	//recv deliver resp
	vector<CGSMPResp *> ::iterator itDelResp;
	CGSMPResp * pDelResp = NULL;
	pthread_mutex_lock(&m_VecDelResp_mutex);
	for(itDelResp = m_pDelResp.begin();itDelResp != m_pDelResp.end(); itDelResp++)
	{
		if((*itDelResp) == NULL)
		{
	   		continue;
		}
		pDelResp = *itDelResp;
		delete (CGSMPResp *)pDelResp;
	}
	m_pDelResp.clear();	
	pthread_mutex_unlock(&m_VecDelResp_mutex);

	//recv submit resp
	vector<CGSMPSubmitResp *> ::iterator itSubResp;
	CGSMPSubmitResp *pSubResp = NULL;
	pthread_mutex_lock(&m_VecSubResp_mutex);
	for(itSubResp = m_pSubResp.begin(); itSubResp != m_pSubResp.end(); itSubResp++)
	{
		if((*itSubResp) == NULL)
		{
	   		continue;
		}
		pSubResp = *itSubResp;
		delete (CGSMPSubmitResp *)pSubResp;
	}
	m_pSubResp.clear();	
	pthread_mutex_unlock(&m_VecSubResp_mutex);

	return;
}

bool CGSMPClient::GetConfig(char *pFile)
{
	if (pFile == NULL)
		return false;

	if (this->m_pXmlConf != NULL)
		delete this->m_pXmlConf;

	this->m_pXmlConf = new CSXmlDoc();
	this->m_pXmlConf->Parse(pFile, 1);

	char cNodeValue[20];
	memset(cNodeValue, 0, sizeof(cNodeValue));
	this->m_pXmlConf->GetItemByName("ServerIP", cNodeValue);
	if (strlen(cNodeValue) == 0)
	{
		cout << "===>no config ServerIP message" << endl;
		delete this->m_pXmlConf;
		this->m_pXmlConf= NULL;
		return false;
	}
	this->m_strIP = cNodeValue;

	memset(cNodeValue, 0, sizeof(cNodeValue));
	this->m_pXmlConf->GetItemByName("ServerPort", cNodeValue);
	if (strlen(cNodeValue) == 0)
	{
		cout << "===>no config ServerPort message" << endl;
		delete this->m_pXmlConf;
		this->m_pXmlConf= NULL;
		return false;
	}
	this->m_nPort = atoi(cNodeValue);

	memset(cNodeValue, 0, sizeof(cNodeValue));
	this->m_pXmlConf->GetItemByName("User", cNodeValue);
	if (strlen(cNodeValue) == 0)
	{
		cout << "===>no config User message" << endl;
		delete this->m_pXmlConf;
		this->m_pXmlConf= NULL;
		return false;
	}
	this->m_strUser = cNodeValue;

	memset(cNodeValue, 0, sizeof(cNodeValue));
	this->m_pXmlConf->GetItemByName("Pass", cNodeValue);
	if (strlen(cNodeValue) == 0)
	{
		cout << "===>no config Pass message" << endl;
		delete this->m_pXmlConf;
		this->m_pXmlConf= NULL;
		return false;
	}
	this->m_strPass = cNodeValue;

	m_bInitConf = true;

	return true;	
}

//װĶӿڣ
bool CGSMPClient::GSMPInit(int nModel, char *pConfFile)
{
	char cFileName[50];
	memset(cFileName, 0, sizeof(cFileName));

	if (pConfFile == NULL)
		strcpy(cFileName, "./client.xml");
	else
		strcpy(cFileName, pConfFile);

	if (!this->GetConfig((char *)cFileName))
		return false;

	if (m_pkg == NULL)
		m_pkg = new CGSMPPkg();

	pthread_attr_t pAttr;
	pthread_attr_init(&pAttr);
	pthread_attr_setdetachstate(&pAttr,  PTHREAD_CREATE_DETACHED);
	pthread_t iThrRecv = 0;

	int ret = pthread_create(&iThrRecv, &pAttr, (void *(*) (void *))CGSMPClient::RecvThread, (void *)this);

	if (ret != 0)
	{
		printf("====>GSMP RECV THREAD ERROR!\n");
		m_bTermitnate = true;
		return false;
	}
	else
		printf("====>GSMP RECV THREAD ID:%ld SUCCEED!\n", iThrRecv);

	m_nModel = nModel;
	return true;
}

void CGSMPClient::GSMPFinal(void)
{
	if (m_nSock > 0)
	{
		DisConnect();
		m_nSock = 0;
	}
	m_bTermitnate = true;
	return;
}

int CGSMPClient::GSMPRecvSubmit(CGSMPSubmit *pSubmit, char *pMsgid, int nTimeOut)
{
	if (pSubmit == NULL)
		return -1;

	time_t tBase = time(NULL);
	vector<CGSMPSubmit * > ::iterator it;

	CGSMPSubmit* pMsg = NULL;

	while (!HadTimeOut(tBase, nTimeOut))
	{
		if (m_pSubmit.size() != 0)
		{
			pthread_mutex_lock(&m_VecSub_mutex);

			for(it = m_pSubmit.begin();it != m_pSubmit.end(); it++)
			{
				if((*it) == NULL)
				{
					m_pSubmit.erase(it);
					it--;
	   				continue;
				}
				pMsg = *it;
				pSubmit->Replace(pMsg);
				m_pSubmit.erase(it);
				delete (CGSMPSubmit *)pMsg;
				break;
			}
			pthread_mutex_unlock(&m_VecSub_mutex);

			return END_SUCCEED;
		}
		else
			usleep(100);
	}

	return END_FAILED;
}

int CGSMPClient::GSMPRecvDeliver(CGSMPDeliver *pDeliver, char *pMsgid, int nTimeOut)
{
	if (pDeliver == NULL)
		return -1;

	time_t tBase = time(NULL);
	vector<CGSMPDeliver * > ::iterator it;

	CGSMPDeliver* pMsg = NULL;

	while (!HadTimeOut(tBase, nTimeOut))
	{
		if (m_pDeliver.size() != 0)
		{
			pthread_mutex_lock(&m_VecDel_mutex);

			for(it = m_pDeliver.begin();it != m_pDeliver.end(); it++)
			{
				if((*it) == NULL)
				{
					m_pDeliver.erase(it);
					it--;
	   				continue;
				}
				pMsg = *it;
				pDeliver->Replace(pMsg);
				m_pDeliver.erase(it);
				delete (CGSMPDeliver *)pMsg;
				break;
			}
			pthread_mutex_unlock(&m_VecDel_mutex);

			return END_SUCCEED;
		}
		else
			usleep(100);
	}

	return END_FAILED;
}

int CGSMPClient::GSMPRecvReport(CGSMPReport *pReport, char *pMsgid, int nTimeOut)
{
	if (pReport == NULL)
		return -1;

	time_t tBase = time(NULL);
	vector<CGSMPReport * > ::iterator it;

	CGSMPReport* pMsg = NULL;

	while (!HadTimeOut(tBase, nTimeOut))
	{
		if (m_pReport.size() != 0)
		{
			pthread_mutex_lock(&m_VecRpt_mutex);

			for(it = m_pReport.begin();it != m_pReport.end(); it++)
			{
				if((*it) == NULL)
				{
					m_pReport.erase(it);
					it--;
	   				continue;
				}
				pMsg = *it;
				pReport->Replace(pMsg);
				m_pReport.erase(it);
				delete (CGSMPReport *)pMsg;
				break;
			}
			pthread_mutex_unlock(&m_VecRpt_mutex);

			return END_SUCCEED;
		}
		else
			usleep(100);
	}

	return END_FAILED;
}


bool CGSMPClient::HadTimeOut(time_t tBase, int nRequestTimeout)
{
	if ((time(NULL) - tBase) >= nRequestTimeout)
		return true;
	else
		return false;
}

int CGSMPClient::GSMPSendSubmit(CGSMPSubmit *pSubmit, char *pMsgid)
{
	char cBuf[5120] = "";

	if (m_nSock <= 0)
	{
		if ((m_nSock = DoConnect(CLIENT_MODEL_SEND)) == END_FAILED)
		{
			printf("=====>GSMP SingleSend DoConnect faild!\n");
			m_nSock = -1;
			pthread_mutex_unlock(&m_send_mutex);
			return END_FAILED;
		}
		else
		{
			printf("=====>GSMP SingleSend DoConnect succeed!\n");
		}
	}

	//
	m_nSubmitTimes = 0;//ͬһSubmitѷ͵Ĵ
	pSubmit->m_Head.nCommandId = GSMP_SUBMIT;
	pSubmit->m_Head.nSequence = m_pkg->GetSeqno();

	while (m_nSubmitTimes < 3)//nSubmitRetry)
	{
		m_nSubmitTimes++;
		cout << "=====>submit send times: " << m_nSubmitTimes << endl;

		memset(&cBuf, 0, sizeof(cBuf));
		int nBufLen = 0;
		m_pkg->DeComposePkg(GSMP_SUBMIT, (void *)pSubmit, (char *)cBuf, &nBufLen);

		while (m_nSock < 0)
		{
			m_nSock = DoConnect(CLIENT_MODEL_SEND);
			if (m_nSock < 0)
			{
				printf("=====>GSMP ====reconnect to the remote host error!=====\n");
				usleep(1000 * 1000 * 5);
			}
		}

		pthread_mutex_lock(&m_send_mutex);
		bool bSendRet = SendToDest(m_nSock, (const char *)&cBuf, (nBufLen));
		pthread_mutex_unlock(&m_send_mutex);
		if ( !bSendRet) 
		{
			//send err, socket error! 
			if (m_nSock > 0)
			{
				close(m_nSock);
				m_nSock = -1;
			}
			
			usleep(1000);
			continue;
		}
		else
		{
			time_t tBase = time(NULL);
			vector<CGSMPSubmitResp * > ::iterator it;
			CGSMPSubmitResp *pSubResp = NULL;

			while ((time(NULL) - tBase) <= 30)
			{
				if (this->m_pSubResp.size() != 0)
				{
					pthread_mutex_lock(&m_VecSubResp_mutex);

					for(it = m_pSubResp.begin();it != m_pSubResp.end(); it++)
					{
						if((*it) == NULL)
						{
							m_pSubResp.erase(it);
							it--;
	   						continue;
						}
						pSubResp = *it;
						if (pSubResp->m_Head.nSequence == pSubmit->m_Head.nSequence)
						{
							m_pSubResp.erase(it);
							int nRet = atoi(pSubResp->m_sResult.c_str());
							memcpy(pMsgid, pSubResp->m_sMsgid.c_str(), pSubResp->m_sMsgid.size());
							delete (CGSMPSubmitResp *)pSubResp;
							pthread_mutex_unlock(&m_VecSubResp_mutex);
							return nRet;
						}
						else if ((time(NULL) - pSubResp->m_nRecvTime) > MAX_RECVRESP_TIME)
						{
							m_pSubResp.erase(it);
							delete (CGSMPResp *)pSubResp;
						}
					}
					pthread_mutex_unlock(&m_VecSubResp_mutex);
				}
				else
					usleep(100);
			}
		}
	}

	return END_FAILED;
}

void *CGSMPClient::RecvThread(CGSMPClient *pClient)
{
	int nRecvLen = 0;
	char cBuf[5120];
	//bool bFirst = true;
	int nLen = 0;

	cout << "=====>START ==== RECV THREAD ID = [" << pthread_self() << "]" << endl;
	while (!pClient->m_bTermitnate)
	{
		while (pClient->m_nSock <= 0)
		{
			if (pClient->m_nModel != CLIENT_MODEL_SEND)
			{
				cout << "======>CLIENT MODEL:" << pClient->m_nModel << endl;
				pClient->m_nSock = pClient->DoConnect(pClient->m_nModel);
				if (pClient->m_nSock <= 0)
					usleep(1000 * 1000 * 10);
			}
			else
				sleep(1);
		}
		
		nRecvLen = sizeof(cBuf);
		memset(cBuf, 0, sizeof(cBuf));
		pthread_mutex_lock(&pClient->m_recv_mutex);
		int nRet = pClient->RecvFromDest(pClient->m_nSock, (char *)cBuf, &nRecvLen, 60);
		pthread_mutex_unlock(&pClient->m_recv_mutex);

		if (nRet == END_SUCCEED)
		{			
			unsigned int nMsgType = 0;

			CGSMPHead mHeadBase;
			pClient->m_pkg->GetPkgHead((const char *)cBuf, mHeadBase);
			nMsgType = mHeadBase.nCommandId;

			if (nMsgType == GSMP_TERMINATE)
			{
				cout << "====>recv terminate ==>sock<==" << pClient->m_nSock << endl;
				break;	
			}
			else if (nMsgType == GSMP_TERMINATE_RESP)
			{
				cout << "====>recv terminate resp==>sock<==" << pClient->m_nSock << endl;
				break;
			}
			else if (nMsgType == GSMP_QLINK)
			{
				cout << "====>recv qlink==>sock<==" << pClient->m_nSock << endl;
				CGSMPHead mHead;
				mHead.nCommandId = GSMP_QLINK_RESP;
				mHead.nTotalLength = HEAD_SIZE;
				mHead.nSequence = mHeadBase.nSequence;
				mHead.nVersion = mHeadBase.nVersion;

				nLen = sizeof(cBuf);
				memset(cBuf, 0, sizeof(cBuf));
				pClient->m_pkg->DeComposePkg(GSMP_QLINK_RESP, (void *)&mHead, (char *)&cBuf, &nLen);
				pthread_mutex_lock(&pClient->m_send_mutex);
				if (!pClient->SendToDest(pClient->m_nSock, (const char *)cBuf, nLen))
				{
					cout << "====>send qlink resp error==>sock<==" << pClient->m_nSock << endl;
					close(pClient->m_nSock);
					pClient->m_nSock = 0;
				}
				pthread_mutex_unlock(&pClient->m_send_mutex);

				continue;
			}
			else if (nMsgType == GSMP_QLINK_RESP)
			{
				cout << "====>recv qlink resp==>sock<==" << pClient->m_nSock << endl;

				continue;
			}
			else if (nMsgType == GSMP_SUBMIT)
			{
				cout << "====>recv submit==>sock<==" << pClient->m_nSock << endl;
				CGSMPSubmit *pSubmit = new CGSMPSubmit();
				CGSMPSubmitResp mSubResp;

				pClient->m_pkg->ComposePkg((void *)pSubmit, (const char *)cBuf, (int *)&nMsgType);
				mSubResp.m_Head.nCommandId = GSMP_SUBMIT_RESP;
				mSubResp.m_Head.nSequence = pSubmit->m_Head.nSequence;
				mSubResp.m_Head.nVersion = pSubmit->m_Head.nVersion;

				pthread_mutex_lock(&pClient->m_VecSub_mutex);
				if (pClient->m_pSubmit.size() < MAX_QUEUE_SIZE)
				{
					pClient->m_pSubmit.push_back(pSubmit);
					mSubResp.m_sResult = "0";
				}
				else
				{
					mSubResp.m_sResult = "11 QUEUE FULL";
					delete pSubmit;
					pSubmit = NULL;
				}
				pthread_mutex_unlock(&pClient->m_VecSub_mutex);

				char cMsgID[100];
				memset(cMsgID, 0, sizeof(cMsgID));
				char cTimeStamp[100];
				memset(cTimeStamp, 0, sizeof(cTimeStamp));
				pClient->m_pkg->GetTimeStamp(time(NULL), (char *)cTimeStamp);
				sprintf(cMsgID, "%s%d", cTimeStamp, pClient->m_pkg->GetSeqno());
				//mSubResp.m_sMsgid = CGSMPGServer::m_strServerID;
				//mSubResp.m_sMsgid += pClient->m_strCPName;
				mSubResp.m_sMsgid += cMsgID;

				//mSubResp.m_sResult = "0";

				nLen = sizeof(cBuf);
				memset(cBuf, 0, sizeof(cBuf));
				pClient->m_pkg->DeComposePkg(GSMP_SUBMIT_RESP, &mSubResp, (char *)&cBuf, &nLen);
				pthread_mutex_lock(&pClient->m_send_mutex);
				if (!pClient->SendToDest(pClient->m_nSock, (const char *)cBuf, nLen))
				{
					close(pClient->m_nSock);
					pClient->m_nSock = 0;
				}
				pthread_mutex_unlock(&pClient->m_send_mutex);

				continue;
			}
			else if (nMsgType == GSMP_SUBMIT_RESP)
			{
				cout << "====>send submit resp==>sock<==" << pClient->m_nSock << endl;
				CGSMPSubmitResp *pSubResp = new CGSMPSubmitResp();
				pClient->m_pkg->ComposePkg((void *)pSubResp, (const char *)cBuf, (int *)&nMsgType);
				pSubResp->m_nRecvTime = time(NULL);
				pthread_mutex_lock(&pClient->m_VecSubResp_mutex);
				pClient->m_pSubResp.push_back(pSubResp);
				pthread_mutex_unlock(&pClient->m_VecSubResp_mutex);

				continue;
			}
			else if (nMsgType == GSMP_DELIVER)
			{
				cout << "====>recv deliver==>sock<==" << pClient->m_nSock << endl;
				CGSMPDeliver *pDeliver = new CGSMPDeliver();
				CGSMPResp mDelResp;
				pClient->m_pkg->ComposePkg((void *)pDeliver, (const char *)cBuf, (int *)&nMsgType);
				mDelResp.m_Head.nCommandId = GSMP_DELIVER_RESP;
				mDelResp.m_Head.nSequence = pDeliver->m_Head.nSequence;
				mDelResp.m_Head.nVersion = pDeliver->m_Head.nVersion;

				pthread_mutex_lock(&pClient->m_VecDel_mutex);
				if (pClient->m_pDeliver.size() < MAX_QUEUE_SIZE)
				{
					pClient->m_pDeliver.push_back(pDeliver);
					mDelResp.m_sResult = "0";
				}
				else
				{
					delete pDeliver;
					pDeliver = NULL;
					mDelResp.m_sResult = "11 QUEUE FULL";
				}
				pthread_mutex_unlock(&pClient->m_VecDel_mutex);
				
				nLen = sizeof(cBuf);
				memset(cBuf, 0, sizeof(cBuf));
				pClient->m_pkg->DeComposePkg(GSMP_DELIVER_RESP, (void *)&mDelResp, (char *)&cBuf, &nLen);
				pthread_mutex_lock(&pClient->m_send_mutex);
				if (!pClient->SendToDest(pClient->m_nSock, (const char *)cBuf, nLen))
				{
					close(pClient->m_nSock);
					pClient->m_nSock = 0;
				}
				pthread_mutex_unlock(&pClient->m_send_mutex);

				continue;
			
			}
			else if (nMsgType == GSMP_DELIVER_RESP)
			{
				cout << "====>send deliver resp==>sock<==" << pClient->m_nSock << endl;
				CGSMPResp *pDelResp = new CGSMPResp();
				pClient->m_pkg->ComposePkg((void *)pDelResp, (const char *)cBuf, (int *)&nMsgType);
				pDelResp->m_nRecvTime = time(NULL);
				pthread_mutex_lock(&pClient->m_VecDelResp_mutex);
				pClient->m_pDelResp.push_back(pDelResp);
				pthread_mutex_unlock(&pClient->m_VecDelResp_mutex);

				continue;
			}
			else if (nMsgType == GSMP_REPORT)
			{
				cout << "====>recv report==>sock<==" << pClient->m_nSock << endl;
				CGSMPReport *pReport = new CGSMPReport();
				CGSMPResp mRptResp;
				pClient->m_pkg->ComposePkg((void *)pReport, (const char *)cBuf, (int *)&nMsgType);
				mRptResp.m_Head.nCommandId = GSMP_REPORT_RESP;
				mRptResp.m_Head.nSequence = pReport->m_Head.nSequence;
				mRptResp.m_Head.nVersion = pReport->m_Head.nVersion;

				pthread_mutex_lock(&pClient->m_VecRpt_mutex);
				if (pClient->m_pReport.size() < MAX_QUEUE_SIZE)
				{
					pClient->m_pReport.push_back(pReport);
					mRptResp.m_sResult = "0";
				}
				else
				{
					delete pReport;
					pReport = NULL;
					mRptResp.m_sResult = "11 QUEUE FULL";
				}
				pthread_mutex_unlock(&pClient->m_VecRpt_mutex);				

				nLen = sizeof(cBuf);
				memset(cBuf, 0, sizeof(cBuf));
				pClient->m_pkg->DeComposePkg(GSMP_REPORT_RESP, &mRptResp, (char *)&cBuf, &nLen);
				pthread_mutex_lock(&pClient->m_send_mutex);
				if (!pClient->SendToDest(pClient->m_nSock, (const char *)cBuf, nLen))
				{
					close(pClient->m_nSock);
					pClient->m_nSock = 0;
				}
				pthread_mutex_unlock(&pClient->m_send_mutex);

				continue;
			
			}
			else if (nMsgType == GSMP_REPORT_RESP)
			{
				cout << "====>send report resp==>sock<==" << pClient->m_nSock << endl;
				CGSMPResp *pRptResp = new CGSMPResp();
				pClient->m_pkg->ComposePkg((void *)pRptResp, (const char *)cBuf, (int *)&nMsgType);
				pRptResp->m_nRecvTime = time(NULL);
				pthread_mutex_lock(&pClient->m_VecRptResp_mutex);
				pClient->m_pRptResp.push_back(pRptResp);
				pthread_mutex_unlock(&pClient->m_VecRptResp_mutex);

				continue;
			}
		}
		else//recv error or time out---break;
		{
			//SEND QLINK
			cout << "====recv time out or error==>sock<==" << pClient->m_nSock << " return = "<< nRet << endl;
			CGSMPHead mHead;
			mHead.nCommandId = GSMP_QLINK;
			mHead.nTotalLength = HEAD_SIZE;
			mHead.nSequence = (unsigned short)pClient->m_pkg->GetSeqno();
			//mHead.nVersion = 1;

			nLen = sizeof(cBuf);
			memset(cBuf, 0, sizeof(cBuf));
			pClient->m_pkg->DeComposePkg(GSMP_QLINK, &mHead, (char *)&cBuf, &nLen);
			cout << "====send qlink==>sock<==" << pClient->m_nSock << endl;
			pthread_mutex_lock(&pClient->m_send_mutex);
			if (!pClient->SendToDest(pClient->m_nSock, (const char *)cBuf, nLen))
			{
				cout << "====send qlink error==>sock<==" << pClient->m_nSock << endl;
				close(pClient->m_nSock);
				pClient->m_nSock = 0;
			}
			pthread_mutex_unlock(&pClient->m_send_mutex);

			//usleep(200);
		}
	}

	cout << "=====>STOP ==== RECV THREAD ID = [" << pthread_self() << "]" << endl;

	return NULL;
}

int CGSMPClient::DoConnect(int nModel)
{
	pthread_mutex_lock(&m_connect_mutex);
	int nSock = ConnectToSMC(nModel); 
	pthread_mutex_unlock(&m_connect_mutex);

	if (nSock > 0)
		printf("DoConnect succeed!\n");
	
	return nSock;
}

int CGSMPClient::ConnectToSMC(int nModel)
 {
	int nSock = 0;
	if (!this->m_bInitConf)
		return -1;

 	for (int i=0; i<3; i++)
 	{
		nSock =  TcpConnect((char *)m_strIP.c_str(),this->m_nPort);
		if (nSock != END_FAILED)
			break;
	}

	if (nSock == END_FAILED)
	{
		printf("TcpConnect error!\n");
		return nSock;
	}
	else
		printf("TcpConnect succeed!\n");

	char cBuf[1024];
	memset(&cBuf, 0, sizeof(cBuf));

	CGSMPLogin mLogin;
	int nBufLen = sizeof(cBuf);
	mLogin.m_Head.nCommandId = GSMP_LOGIN;
	mLogin.m_Head.nSequence = m_pkg->GetSeqno();
	mLogin.m_sUser = this->m_strUser;
	mLogin.m_sPassword = this->m_strPass;

	char cModel[10];
	memset(cModel, 0, sizeof(cModel));
	sprintf(cModel, "%02d", nModel);
	mLogin.m_sLoginType = cModel;

	m_pkg->DeComposePkg(GSMP_LOGIN, (void *)&mLogin, (char *)cBuf, &nBufLen);

	if (nBufLen == 0)
		return END_FAILED;
	
	if ( SendToDest(nSock, (const char *)&cBuf, nBufLen) )
	{
		memset(&cBuf, 0, sizeof(cBuf));
		int nRcvMsgLen = sizeof(cBuf);
		int nRet = -1;
		nRet = RecvFromDest(nSock, (char *)&cBuf, &nRcvMsgLen, 60);
		if ((nRet == END_SUCCEED) && (nRcvMsgLen >= HEAD_SIZE))
		{
			CGSMPResp mLoginResp;
			m_pkg->GetPkgHead((const char *)cBuf, mLoginResp.m_Head);
			if (mLoginResp.m_Head.nCommandId != GSMP_LOGIN_RESP)
			{
				close(nSock);
				nSock = -1;
				return END_FAILED;
			}

			int nPakcageType = 0;
			m_pkg->ComposePkg((void *)&mLoginResp, (const char *)cBuf, &nPakcageType);

			int nStatus = atoi(mLoginResp.m_sResult.c_str());
			if ((nStatus == 0) && (mLoginResp.m_sResult.size() != 0))
			{
				return nSock;
			}
			else
			{
				close(nSock);
				nSock = -1;
				return END_FAILED;
			}
		}
		else
			printf("recv error!\n");
	}
	else
		printf("Send login mesg error!\n");

	//error!
	close(nSock);
	nSock = -1;
	return END_FAILED;
 } 
 
int CGSMPClient::DisConnect(void)
 {
	if (this->m_nSock <= 0)
		return END_FAILED;

	char cBuf[1024];
	memset(&cBuf, 0, sizeof(cBuf));

	CGSMPHead mHead;
	mHead.nCommandId = GSMP_TERMINATE;
	mHead.nSequence = this->m_pkg->GetSeqno();

	int nLen = sizeof(cBuf);
	m_pkg->DeComposePkg(GSMP_TERMINATE, (void *)&mHead, (char *)&cBuf, &nLen);

	SendToDest(m_nSock, (const char *)&cBuf, nLen);
	close(m_nSock);
	m_nSock = 0;
	return END_SUCCEED;
 } 



