package com.wisentsoft.isms;

import com.wisentsoft.isms.utils.HttpUtils;
import com.wisentsoft.isms.utils.JsonUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * Hello world!
 */
public class App {
    private static final String sendApi = "https://dx.11185.cn:13011/isms-send-api";

    private static final Logger log = LoggerFactory.getLogger(App.class);

    private static final String appId = "<appKey>";//appKey
    private static final String secret = "<appSecret>";//密钥

    private static Map<String, String> getHeads() {
        Map<String, String> heads = new HashMap<>();
        heads.put("appKey", appId);
        heads.put("appSecret", secret);
        return heads;
    }

    public static void main(String[] args) {
        System.out.println("Hello World!");

        HttpUtils.sslSafe = false;

        //测试单条发送
        sendSms();

        //测试批量发送
        sendBatchSms();

        //测试批量发送
        sendBatchSmsJson();
    }

    /**
     * 发送单条短信
     */
    private static void sendSms() {

        //测试数据
        String templateId = "";//请填写短信模板ID，短信模板ID在控制台上面获取
        String signId = "";//请填写短信签名ID，短信签名ID可以在控制台上面获取
        String phone = "";//短信接收手机号码, 只能填写单个手机号码

        String url = String.format(Locale.CHINA, "%s/send/sms", sendApi);//单条发送地址

        log.info("sendApi - {}", url);

        Map<String, String> values = new HashMap<>();
        values.put("name", "余");//模板变量

        String valuesJsonStr = JsonUtils.toJsonStr(values);

        List<NameValuePair> list = new ArrayList<>();

        list.add(new BasicNameValuePair("templateId", templateId));
        list.add(new BasicNameValuePair("signId", signId));
        list.add(new BasicNameValuePair("phone", phone));
        list.add(new BasicNameValuePair("varValues", valuesJsonStr));

        UrlEncodedFormEntity urlEncodedFormEntity = new UrlEncodedFormEntity(list, StandardCharsets.UTF_8);

        String result = HttpUtils.post(url, getHeads(), urlEncodedFormEntity);
        log.debug("http result - {}", result);
        if (StringUtils.isNotBlank(result)) {
            log.debug("http result pretty - \n\n{}\n\n", JsonUtils.jsonStrToPrettyJsonStr(result));
        }

    }//end method

    /**
     * 发送批量短信，批量手机号和批量模板参数使用form field同名name数组
     */
    public static void sendBatchSms() {

        //测试数据
        String templateId = "";//请填写短信模板ID，短信模板ID在控制台上面获取
        String signId = "";//请填写短信签名ID，短信签名ID可以在控制台上面获取
        String[] phoneArray = new String[]{"186****2938", "186****2937"};//接收短信的手机号码数组

        Map<String, String> values1 = new HashMap<>();
        values1.put("name", "张三");//模板变量

        String values1JsonStr = JsonUtils.toJsonStr(values1);

        Map<String, String> values2 = new HashMap<>();
        values2.put("name", "李四");//模板变量

        String values2JsonStr = JsonUtils.toJsonStr(values2);

        String[] varValuesArray = new String[]{values1JsonStr, values2JsonStr};

        String url = String.format(Locale.CHINA, "%s/send/batchSms", sendApi);//批量发送地址

        log.info("sendApi - {}", url);

        ContentType APPLICATION_FORM_URLENCODED = ContentType.APPLICATION_FORM_URLENCODED.withCharset(StandardCharsets.UTF_8);
        MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create();
        entityBuilder.addTextBody("templateId", templateId, APPLICATION_FORM_URLENCODED);
        entityBuilder.addTextBody("signId", signId, APPLICATION_FORM_URLENCODED);
        //有多少个手机号就添加多少个name为phoneArray的form field，每个field value = 手机号
        Arrays.asList(phoneArray).forEach(phone -> entityBuilder.addTextBody("phoneArray", phone, APPLICATION_FORM_URLENCODED));
        //有多少个varValues就添加多少个name为varValuesArray的form field，每个field value = varValues字符串
        Arrays.asList(varValuesArray).forEach(varValues -> entityBuilder.addTextBody("varValuesArray", varValues, APPLICATION_FORM_URLENCODED));

        HttpEntity httpEntity = entityBuilder.build();

        String result = HttpUtils.post(url, getHeads(), httpEntity);
        log.debug("http result - {}", result);
        if (StringUtils.isNotBlank(result)) {
            log.debug("http result pretty - \n\n{}\n\n", JsonUtils.jsonStrToPrettyJsonStr(result));
        }

    }//end method

    /**
     * 批量发送短信，批量手机号和和批量模板参数使用json数组方式
     */
    public static void sendBatchSmsJson() {

        //测试数据
        String templateId = "";//请填写短信模板ID，短信模板ID在控制台上面获取
        String signId = "";//请填写短信签名ID，短信签名ID可以在控制台上面获取
        String[] phoneArray = new String[]{"186****2938", "186****2937"};//接收短信的手机号码数组

        String phoneJsonArray = JsonUtils.toJsonStr(phoneArray);
        log.debug("phoneJsonArray - {}", phoneJsonArray);

        Map<String, String> values1 = new HashMap<>();
        values1.put("name", "张三");//模板变量
        values1.put("age", "18");//模板变量
        String values1JsonStr = JsonUtils.toJsonStr(values1);

        Map<String, String> values2 = new HashMap<>();
        values2.put("name", "李四");//模板变量
        values2.put("age", "19");//模板变量

        String values2JsonStr = JsonUtils.toJsonStr(values2);

        String[] varValuesArray = new String[]{values1JsonStr, values2JsonStr};//存放到字符串数组中
        String varValuesJsonArray = JsonUtils.toJsonStr(varValuesArray);
        log.debug("varValuesJsonArray - {}", varValuesJsonArray);

        String url = String.format(Locale.CHINA, "%s/send/batchSms", sendApi);//批量发送地址

        log.info("sendApi - {}", url);

        List<NameValuePair> list = new ArrayList<>();

        list.add(new BasicNameValuePair("templateId", templateId));
        list.add(new BasicNameValuePair("signId", signId));
        list.add(new BasicNameValuePair("phoneJsonArray", phoneJsonArray));
        list.add(new BasicNameValuePair("varValuesJsonArray", varValuesJsonArray));

        UrlEncodedFormEntity urlEncodedFormEntity = new UrlEncodedFormEntity(list, StandardCharsets.UTF_8);

        String result = HttpUtils.post(url, getHeads(), urlEncodedFormEntity);

        log.debug("http result - {}", result);
        if (StringUtils.isNotBlank(result)) {
            log.debug("http result pretty - \n\n{}\n\n", JsonUtils.jsonStrToPrettyJsonStr(result));
        }

    }//end method
}
