package com.wisentsoft.isms.utils;

import org.apache.commons.codec.CharEncoding;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;

public class HttpUtils {
    private static final Logger log = LoggerFactory.getLogger(HttpUtils.class);

    /**
     * 设置访问SSL连接地址，是否校验SSL证书。根据实际情况设置
     */
    public static boolean sslSafe = false;

    public static String post(String url, Map<String, String> heads, HttpEntity httpEntity) {
        HttpPost httppost = new HttpPost(url);

        if (null != heads && !heads.isEmpty()) {
            heads.forEach(httppost::setHeader);
        }

        httppost.setEntity(httpEntity);

        CloseableHttpClient httpClient;
        if (StringUtils.startsWith(url, "https://")) {
            httpClient = sslSafe ? createDefaultHttpClient() : createNotSafeHttpClient();
        } else {
            httpClient = createDefaultHttpClient();
        }

        if (null == httpClient) {
            return null;
        }

        CloseableHttpResponse response = null;
        try {
            response = httpClient.execute(httppost);
            StatusLine statusLine = response.getStatusLine();
            Header[] headers = response.getAllHeaders();
            for (Header header: headers){
                log.debug("head: {} : {}", header.getName(), header.getValue());
            }


            if (HttpStatus.SC_OK != statusLine.getStatusCode()) {
                log.error("接口返回错误响应码：{}, {}", statusLine.getStatusCode(), statusLine.getReasonPhrase());
                return null;
            }



            HttpEntity entity = response.getEntity();
            if (null == entity) {
                log.error("调用发送接口失败");
                return null;
            }

            return EntityUtils.toString(entity, CharEncoding.UTF_8);
        } catch (Exception e) {
            log.error("调用发送接口出现异常", e);
        } finally {
            if (null != response) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            try {
                httpClient.close();
            } catch (Exception e) {
                log.error("接口异常", e);
            }
        }

        return null;
    }

    private static CloseableHttpClient createDefaultHttpClient() {
        return HttpClients.createDefault();
    }

    private static CloseableHttpClient createNotSafeHttpClient() {
        try {
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
                @Override
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            }).build();
            SSLConnectionSocketFactory sslConnectionSocketFactory = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);
            return HttpClients.custom().setSSLSocketFactory(sslConnectionSocketFactory).build();
        } catch (NoSuchAlgorithmException | KeyStoreException | KeyManagementException e) {
            log.error("创建 httpClient fail", e);
            return null;
        }

    }
}
